#!/bin/bash


print_usage() {
	echo "Usage: run.sh [ <jvm-options> -- ] <bugmaps-options>" >&2
	echo "Options:" >&2
	echo "	-r, --repo		URL for the repository (can be local folder)" >&2
	echo "	-b, --bugs_file		Path to the issue tracker file" >&2
	echo "	-s, --start		Date to start the analysis  (yyyy-mm-dd)" >&2
	echo "	-e, --end		Date to finish the analysis (yyyy-mm-dd)" >&2
	echo "	-g, --granger		Generate data to run the Granger causality test" >&2
	echo
}



lowercase(){
    echo "$1" | sed "y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/"
}



findOS() {
    OS=$(lowercase $(uname))
    KERNEL=$(uname -r)
    MACH=$(uname -m)

    if [ "{$OS}" == "windowsnt" ]; then
        OS=windows
    elif [ $OS == darwin ]; then
        OS=mac
    else
        OS=$(uname)
        if [ "${OS}" = "SunOS" ] ; then
            OS=Solaris
            ARCH=$(uname -p)
            OSSTR="${OS} ${REV}(${ARCH} $(uname -v))"
        elif [ "${OS}" = "AIX" ] ; then
            OSSTR="${OS} $(oslevel) ($(oslevel -r))"
        elif [ "${OS}" = "Linux" ] ; then
            if [ -f /etc/redhat-release ] ; then
                DistroBasedOn='RedHat'
                DIST=$(cat /etc/redhat-release |sed s/\ release.*//)
                PSUEDONAME=$(cat /etc/redhat-release | sed s/.*\(// | sed s/\)//)
                REV=$(cat /etc/redhat-release | sed s/.*release\ // | sed s/\ .*//)
            elif [ -f /etc/SuSE-release ] ; then
                DistroBasedOn='SuSe'
                PSUEDONAME=$(cat /etc/SuSE-release | tr "\n" ' '| sed s/VERSION.*//)
                REV=$(cat /etc/SuSE-release | tr "\n" ' ' | sed s/.*=\ //)
            elif [ -f /etc/mandrake-release ] ; then
                DistroBasedOn='Mandrake'
                PSUEDONAME=$(cat /etc/mandrake-release | sed s/.*\(// | sed s/\)//)
                REV=$(cat /etc/mandrake-release | sed s/.*release\ // | sed s/\ .*//)
            elif [ -f /etc/debian_version ] ; then
                DistroBasedOn='Debian'
                DIST=$(cat /etc/lsb-release | grep '^DISTRIB_ID' | awk -F=  '{ print $2 }')
                PSUEDONAME=$(cat /etc/lsb-release | grep '^DISTRIB_CODENAME' | awk -F=  '{ print $2 }')
                REV=$(cat /etc/lsb-release | grep '^DISTRIB_RELEASE' | awk -F=  '{ print $2 }')
            fi
            if [ -f /etc/UnitedLinux-release ] ; then
                DIST="${DIST}[$(cat /etc/UnitedLinux-release | tr "\n" ' ' | sed s/VERSION.*//)]"
            fi
        fi
    fi

    OS=$(lowercase $OS)
    DistroBasedOn=$(lowercase $DistroBasedOn)
    readonly OS
    readonly DIST
    readonly DistroBasedOn
    readonly PSUEDONAME
    readonly REV
    readonly KERNEL
    readonly MACH

    #echo "OS = $OS"
    #echo "DIST = $DIST"
    #echo "REV = $REV"
    #echo "PSUEDONAME = $PSUEDONAME"
    #echo "DistroBasedOn = $DistroBasedOn"
    #echo "KERNEL = $KERNEL"
    #echo "MACH = $MACH"
}




# Function copied and adapted from Freemind (http://freemind.sourceforge.net)
findjava() {
	# We try hard to find the proper 'java' command
	if [ -n "${JAVACMD}" ] && [ -x "${JAVACMD}" ]
	then
		echo "Using \$JAVACMD to find java virtual machine."
	elif [ -n "${JAVA_BINDIR}" ] && [ -x "${JAVA_BINDIR}/java" ]
	then
		JAVACMD="${JAVA_BINDIR}/java"
		#_debug "Using \$JAVA_BINDIR to find java virtual machine."
	elif [ -n "${JAVA_HOME}" ] && [ -x "${JAVA_HOME}/bin/java" ]
	then
		JAVACMD="${JAVA_HOME}/bin/java"
		#_debug "Using \$JAVA_HOME to find java virtual machine."
	else
		JAVACMD=$(which java)
		if [ -n "${JAVACMD}" ] && [ -x "${JAVACMD}" ]
		then
			echo "Using \$PATH to find java virtual machine."
		elif [ -x /usr/bin/java ]
		then
			#_debug "Using /usr/bin/java to find java virtual machine."
			JAVACMD=/usr/bin/java
		fi
	fi

	# if we were successful, we return 0 else we complain and return 1
	if [ -n "${JAVACMD}" ] && [ -x "${JAVACMD}" ]
	then
		return 0
	else
		echo "Couldn't find a java virtual machine, define JAVACMD, JAVA_BINDIR, JAVA_HOME or PATH."
		return 1
	fi
}




getParameters() {
	# JVM options e.g. -Xmx2500m to augment maximum memory size of the vm to 2.5Go.
	JOPT=""
	# BugMaps options
	BMOPT=""

	# Any argument before "--" is for the JVM
	# Any argument after "--" is for BugMaps
	# Without "--" every argument goes to BugMaps
	while [ ${1:-"--"} != "--" ]
	do
		JOPT="$JOPT $1"
		shift
	done
	if [ "$1" == "--" ]
	then
		shift
		BMOPT=$*
	else
		# without the special "--" argument, all options are assumed to be for BugMaps
		BMOPT=$JOPT
		JOPT="-Xmx1024m"
	fi
	parseBugmapsParameters $BMOPT
}




parseBugmapsParameters() {
	granger=false
	while true
	do
		case "$1" in
			-r | --repo)
				repo="$2"   # You may want to check validity of $2
				shift 2
				;;
			-b | --bugs)
				bugs_file="$2"   # You may want to check validity of $2
				shift 2
				;;
			-s | --start)
				start_date="$2" # You may want to check validity of $2
				shift 2
				;;
			-e | --end)
				end_date="$2" # You may want to check validity of $2
				shift 2
				;;
			-g | --granger)
				granger=granger
				shift 1
				;;
			-h | --help)
				print_usage
				exit 0
				;;
			-*)
				echo "Error: Unknown option: $1" >&2
				print_usage
				exit 1
				;;
			*)  # No more options
				break
				;;
		esac
	done
	BMOPT="$repo $bugs_file $start_date $end_date $granger"
}





## script starts here! ##

# Looking for 'java' command
findjava
if [ $? -ne 0 ]
then
	exit 1
fi


# Processing command line parameters
getParameters $*

#echo "JVM parameters:"
#echo "  $JOPT"
#echo "BugMaps parameters:"
#echo "  $BMOPT"

echo "Starting BugMaps Importer ..."
"${JAVACMD}" $JOPT -jar bugmaps-importer.jar $BMOPT



# Finding wich OS we are running to boot the proper moose executable
findOS
MOOSE_PATH=Moose.app
echo "OS="
case $OS in
    linux)   MOOSECMD=$MOOSE_PATH/Contents/Linux/squeak
    		 MOOSEARGS="$MOOSE_PATH/Contents/Resources/Moose.image install_bugmaps.st"
    		 ;;
    mac)     MOOSECMD="$MOOSE_PATH/Contents/MacOS/'Squeak VM Opt'"
    		 MOOSEARGS="$MOOSE_PATH/Contents/Resources/Moose.image install_bugmaps.st"
    		 ;;
    windows) MOOSECMD=$MOOSE_PATH/Contents/Windows/Squeak.exe
    		 MOOSEARGS="install_bugmaps.st"
    		 ;;
    cygwin)  MOOSECMD=$MOOSE_PATH/Contents/Windows/Squeak.exe
    		 MOOSEARGS="install_bugmaps.st"
    		 ;;
esac


echo "Starting BugMaps ..."
echo "$MOOSECMD $MOOSEARGS"
eval $MOOSECMD $MOOSEARGS
exit 0

